@@ -130,7 +130,7 @@ static void mncc_fill_header(struct gsm_mncc *mncc, uint32_t msg_type, uint32_t
 	}
 }
 
-static void mncc_write(struct mncc_connection *conn, struct gsm_mncc *mncc, uint32_t callref)
+static int mncc_write(struct mncc_connection *conn, struct gsm_mncc *mncc)
 {
 	int rc;
 
@@ -141,32 +141,43 @@ static void mncc_write(struct mncc_connection *conn, struct gsm_mncc *mncc, uint
 	rc = write(conn->fd.fd, mncc, sizeof(*mncc));
 	LOGP(DMNCC, LOGL_DEBUG, "MNCC sent message type: %s\n", osmo_mncc_name(mncc->msg_type));
 	if (rc != sizeof(*mncc)) {
-		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u)\n", callref);
+		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u)\n", mncc->callref);
 		close_connection(conn);
 	}
+
+	return rc;
 }
 
-static void mncc_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref)
+static int mncc_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref)
 {
 	struct gsm_mncc mncc = { 0, };
 
 	mncc_fill_header(&mncc, msg_type, callref);
-	mncc_write(conn, &mncc, callref);
+	return mncc_write(conn, &mncc);
 }
 
-static void mncc_rtp_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref)
+static int mncc_rtp_write(struct mncc_connection *conn, struct gsm_mncc_rtp *rtp)
 {
 	int rc;
+
+	rc = write(conn->fd.fd, rtp, sizeof(*rtp));
+	LOGP(DMNCC, LOGL_DEBUG, "MNCC sent message type: %s\n", osmo_mncc_name(rtp->msg_type));
+	if (rc != sizeof(*rtp)) {
+		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u): %d\n", rtp->callref, rc);
+		close_connection(conn);
+	}
+
+	return rc;
+}
+
+static int mncc_rtp_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref)
+{
 	struct gsm_mncc_rtp mncc = { 0, };
 
 	mncc.msg_type = msg_type;
 	mncc.callref = callref;
 
-	rc = write(conn->fd.fd, &mncc, sizeof(mncc));
-	if (rc != sizeof(mncc)) {
-		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u)\n", callref);
-		close_connection(conn);
-	}
+	return mncc_rtp_write(conn, &mncc);
 }
 
 /* Send a MNCC_RTP_CONNECT to the MSC for the given call legs */
@@ -174,7 +185,7 @@ static bool send_rtp_connect(struct mncc_call_leg *leg, struct call_leg *other)
 {
 	struct gsm_mncc_rtp mncc = { 0, };
 	int rc;
-	char ip_addr[INET_ADDRSTRLEN];
+	char ip_addr[INET6_ADDRSTRLEN];
 
 	/*
 	 * Send RTP CONNECT and we handle the general failure of it by
@@ -182,17 +193,16 @@ static bool send_rtp_connect(struct mncc_call_leg *leg, struct call_leg *other)
 	 */
 	mncc.msg_type = MNCC_RTP_CONNECT;
 	mncc.callref = leg->callref;
-	mncc.ip = ntohl(other->ip);
-	mncc.port = other->port;
+	mncc.addr = other->addr;
 	mncc.payload_type = other->payload_type;
 	/*
 	 * FIXME: mncc.payload_msg_type should already be compatible.. but
 	 * payload_type should be different..
 	 */
-	struct in_addr net = { .s_addr = other->ip };
-	inet_ntop(AF_INET, &net, ip_addr, sizeof(ip_addr));
-	LOGP(DMNCC, LOGL_DEBUG, "SEND rtp_connect: IP=(%s) PORT=(%u)\n", ip_addr, mncc.port);
-	rc = write(leg->conn->fd.fd, &mncc, sizeof(mncc));
+	LOGP(DMNCC, LOGL_DEBUG, "SEND rtp_connect: IP=(%s) PORT=(%u)\n",
+	     osmo_sockaddr_ntop((const struct sockaddr*)&other->addr, ip_addr),
+	     osmo_sockaddr_port((const struct sockaddr*)&other->addr));
+	rc = mncc_rtp_write(leg->conn, &mncc);
 	if (rc != sizeof(mncc)) {
 		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u)\n",
 			leg->callref);
@@ -265,14 +275,15 @@ static void mncc_call_leg_ring(struct call_leg *_leg)
 	out_mncc.progress.location = GSM48_CAUSE_LOC_PRN_S_LU; /* Private network serving the local user */
 	out_mncc.progress.descr = GSM48_PROGR_IN_BAND_AVAIL; /* In-band information or appropriate pattern now available */
 
-	mncc_write(leg->conn, &out_mncc, leg->callref);
+	mncc_write(leg->conn, &out_mncc);
 
 	/*
 	 * If we have remote IP/port let's connect it already.
 	 * FIXME: We would like to keep this as recvonly...
 	 */
 	other_leg = call_leg_other(&leg->base);
-	if (other_leg && other_leg->port != 0 && other_leg->ip != 0)
+	if (other_leg && other_leg->addr.ss_family != AF_UNSPEC &&
+	    osmo_sockaddr_port((const struct sockaddr *)&other_leg->addr) != 0)
 		send_rtp_connect(leg, other_leg);
 }
 
@@ -393,11 +404,14 @@ static void check_rtp_connect(struct mncc_connection *conn, const char *buf, int
 	leg = mncc_find_leg_not_released(rtp->callref);
 	if (!leg) {
 		LOGP(DMNCC, LOGL_ERROR, "leg(%u) can not be found\n", rtp->callref);
-		return mncc_send(conn, MNCC_REJ_REQ, rtp->callref);
+		mncc_send(conn, MNCC_REJ_REQ, rtp->callref);
+		return;
 	}
 
 	/* extract information about where the RTP is */
-	if (rtp->ip != 0 || rtp->port != 0 || rtp->payload_type != 0)
+	if (rtp->addr.ss_family != AF_UNSPEC ||
+	    osmo_sockaddr_port((const struct sockaddr *)&rtp->addr) != 0 ||
+	    rtp->payload_type != 0)
 		return;
 
 	LOGP(DMNCC, LOGL_ERROR, "leg(%u) rtp connect failed\n", rtp->callref);
@@ -412,7 +426,7 @@ static void check_rtp_create(struct mncc_connection *conn, const char *buf, int
 {
 	const struct gsm_mncc_rtp *rtp;
 	struct mncc_call_leg *leg;
-	char ip_addr[INET_ADDRSTRLEN];
+	char ip_addr[INET6_ADDRSTRLEN];
 
 	if (rc < sizeof(*rtp)) {
 		LOGP(DMNCC, LOGL_ERROR, "gsm_mncc_rtp of wrong size %d < %zu\n",
@@ -424,21 +438,21 @@ static void check_rtp_create(struct mncc_connection *conn, const char *buf, int
 	leg = mncc_find_leg_not_released(rtp->callref);
 	if (!leg) {
 		LOGP(DMNCC, LOGL_ERROR, "call(%u) can not be found\n", rtp->callref);
-		return mncc_send(conn, MNCC_REJ_REQ, rtp->callref);
+		mncc_send(conn, MNCC_REJ_REQ, rtp->callref);
+		return;
 	}
 
 	/* extract information about where the RTP is */
-	leg->base.ip = htonl(rtp->ip);
-	leg->base.port = rtp->port;
+	leg->base.addr = rtp->addr;
 	leg->base.payload_type = rtp->payload_type;
 	leg->base.payload_msg_type = rtp->payload_msg_type;
 
 	/* TODO.. now we can continue with the call */
-	struct in_addr net = { .s_addr = leg->base.ip };
-	inet_ntop(AF_INET, &net, ip_addr, sizeof(ip_addr));
 	LOGP(DMNCC, LOGL_INFO,
 		"RTP continue leg(%u) ip(%s), port(%u) pt(%u) ptm(%u)\n",
-		leg->callref, ip_addr, leg->base.port,
+		leg->callref,
+		osmo_sockaddr_ntop((const struct sockaddr*)&leg->base.addr, ip_addr),
+		osmo_sockaddr_port((const struct sockaddr*)&leg->base.addr),
 		leg->base.payload_type, leg->base.payload_msg_type);
 	stop_cmd_timer(leg, MNCC_RTP_CREATE);
 	continue_call(leg);
@@ -487,7 +501,8 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 			LOGP(DMNCC, LOGL_ERROR,
 				"MNCC leg(%u) without called addr fields(%u)\n",
 				data->callref, data->fields);
-			return mncc_send(conn, MNCC_REJ_REQ, data->callref);
+			mncc_send(conn, MNCC_REJ_REQ, data->callref);
+			return;
 		}
 
 		/* Emergency without a called number present. Use the standard "emergency" number. */
@@ -498,14 +513,16 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 		LOGP(DMNCC, LOGL_ERROR,
 			"MNCC leg(%u) without calling addr fields(%u)\n",
 			data->callref, data->fields);
-		return mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		return;
 	}
 
 	/* TODO.. bearer caps and better audio handling */
 	if (!continue_setup(conn, data)) {
 		LOGP(DMNCC, LOGL_ERROR,
 			"MNCC screening parameters failed leg(%u)\n", data->callref);
-		return mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		return;
 	}
 
 	/* Create an RTP port and then allocate a call */
@@ -513,7 +530,8 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 	if (!call) {
 		LOGP(DMNCC, LOGL_ERROR,
 			"MNCC leg(%u) failed to allocate call\n", data->callref);
-		return mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		mncc_send(conn, MNCC_REJ_REQ, data->callref);
+		return;
 	}
 
 	leg = (struct mncc_call_leg *) call->initial;
@@ -806,7 +824,7 @@ static void check_dtmf_start(struct mncc_connection *conn, const char *buf, int
 	mncc_fill_header(&out_mncc, MNCC_START_DTMF_RSP, leg->callref);
 	out_mncc.fields |= MNCC_F_KEYPAD;
 	out_mncc.keypad = data->keypad;
-	mncc_write(conn, &out_mncc, leg->callref);
+	mncc_write(conn, &out_mncc);
 }
 
 static void check_dtmf_stop(struct mncc_connection *conn, const char *buf, int rc)
@@ -824,7 +842,7 @@ static void check_dtmf_stop(struct mncc_connection *conn, const char *buf, int r
 	mncc_fill_header(&out_mncc, MNCC_STOP_DTMF_RSP, leg->callref);
 	out_mncc.fields |= MNCC_F_KEYPAD;
 	out_mncc.keypad = data->keypad;
-	mncc_write(conn, &out_mncc, leg->callref);
+	mncc_write(conn, &out_mncc);
 }
 
 static void check_hello(struct mncc_connection *conn, const char *buf, int rc)
@@ -904,11 +922,10 @@ int mncc_create_remote_leg(struct mncc_connection *conn, struct call *call)
 	 *  - Screening, redirect?
 	 *  - Synth. the bearer caps based on codecs?
 	 */
-	rc = write(conn->fd.fd, &mncc, sizeof(mncc));
+	rc = mncc_write(conn, &mncc);
 	if (rc != sizeof(mncc)) {
 		LOGP(DMNCC, LOGL_ERROR, "Failed to send message leg(%u)\n",
 			leg->callref);
-		close_connection(conn);
 		talloc_free(leg);
 		return -1;
 	}
